/* common.c - Common functions */

#include "common.h"
#include "debug.h"

void print_help (void) {
	printf("\nGeneral options:\n");
	printf("--help           -h         Show help text (this text)\n");
	printf("--version        -v         Show SL45c version information\n");
	printf("\nFunctional options:\n");
	printf("--info           -i         Information about connected phone\n");
	printf("--scan           -S         Scan for available networks\n");
	printf("--last-dialed    -l         Show dialed numbers\n");
	printf("--last-received  -r         Show received calls\n");
	printf("--missed-calls   -m         Show missed calls\n");
	printf("--synctime       -t         Sync time on GSM to system's clock\n");
	printf("--smenu #                   Enable the Service Menu in the phone\n");
	printf("                            # = 0 to disable Service Menu\n");
	printf("                            # = 1 for Full Service Menu\n");
	printf("                            # = 2 for Netmonitor / Audiomonitor\n");
	printf("--read (bmp,mid,vcf,vcs)    Read selected memory to a file\n");
	printf("--write (bmp,mid,vcf,vcs)   Write file to selected memory\n");
	printf("--dump                      Dump Addressbook entries to STDOUT\n");
	printf("--sim            -s         Dump SIM-stored addressbook to STDOUT\n");
	printf("\nConfiguration options:\n");
	printf("--in [filename]             Use [filename] for input (when needed)\n");
	printf("--out [filename]            Write data to [filename]\n");
	printf("--rmempos [number]			Read memory position [number]\n");
	printf("--wmempos [number]			Write memory position [number]\n");
	printf("--csv                       Output --dump in CSV format\n");
	printf("--debug ...      -D ...     Set debug level to ...\n");
	printf("--device ...     -d ...     Use device ...\n");
	printf("--speed ...      -e ...     Set serial port speed to ...\n");
}

void print_version (void) {
	printf("\n");
	printf("Running version: %s\n", VERSION);
	printf("Build date:      %s\n", DATE);
	printf("Built by:        %s@%s\n", USER, HOST);
	printf("Compiler used:   %s\n", COMPILER);
	printf("Flags:           %s\n", FLAGS);
	exit (0);
}

struct s_result rxp (const char *p, const char *data) {
	struct s_result retval;		/* Holds results */
	int retvalc = 0;
    pcre *re = NULL;
    const char *error;
    int erroroffset, count, i, start_offset = 0;
	int options = 0;
    int len = strlen(data);
    int offsets[128];
    int size_offsets = sizeof(offsets)/sizeof(int);

	retval.result = calloc((unsigned int)size_offsets, 1);

    re = pcre_compile(p, options, &error, &erroroffset, NULL);
    if (re == NULL) {
        printf("Regexp Compile Failed: %s at offset %d\n", error, erroroffset);
        return retval;
    }

	count = pcre_exec(re, NULL, data, len, start_offset, options, offsets, size_offsets);
    while (count > 0) {
        for (i = 0; i < count*2; i += 2) {
            if (offsets[i] >= 0) {
			    char *res;

                res = malloc((offsets[i+1] - offsets[i]) * sizeof(char)+1);
                pmesg(100, "SO: %d\tEO: %d\tDF: %d\t", offsets[i], offsets[i+1], offsets[i+1] - offsets[i]);
                strncpy(res, data+offsets[i], (unsigned int)offsets[i+1] - offsets[i]);
                res[offsets[i+1]-offsets[i]] = 0;
                retval.result[retvalc] = res;
                pmesg(100, "res: %s\n",retval.result[retvalc]);
                retvalc++;
            }
        }
		count = pcre_exec(re, NULL, data, len, offsets[1], options, offsets, size_offsets);
    }

	retval.elemcount = retvalc;
    return retval;
}

void strip (char *where) {
	int counter = 0;
	for (counter = 0; counter <= strlen(where); counter++) {
		if (where[counter] == '\n' || where[counter] == '\r')
			where[counter] = 0;
	}
}
	
