/* serial.c - Handles Serial Communication */

#include "common.h"
#include "debug.h"

void ser_open (void) {
	int flags = O_RDWR | O_NOCTTY | O_NONBLOCK;
	struct termios ser_tio;

	pmesg(100, "Opening %s...\n", tasks.device);

	tasks.ser_con = open(tasks.device, flags);
	if (tasks.ser_con == -1) {
		perror(tasks.device);
		exit (1);
	}

	fcntl(tasks.ser_con, F_SETFL, O_RDWR);

	bzero(&ser_tio, sizeof(ser_tio));
	cfsetispeed(&ser_tio, tasks.speed);
	cfsetospeed(&ser_tio, tasks.speed);
	ser_tio.c_cflag |= (CS8 | CLOCAL | CREAD);
	ser_tio.c_cflag &= ~PARENB;
	ser_tio.c_cflag &= ~CSTOPB;
	ser_tio.c_lflag |= (ICANON);
	ser_tio.c_cflag |= CRTSCTS;
	tcflush(tasks.ser_con, TCIOFLUSH);
	tcsetattr(tasks.ser_con, TCSANOW, &ser_tio);
}

void ser_send (const char *data) {
	pmesg(100, "Sending %s...\n", data);
	if ( (write(tasks.ser_con, data, strlen(data)) != strlen(data)) ) /* || (write(ser_con, "\r", 1) != 1) ) */
		printf("Error sending data... (%s) \n", strerror(errno));
}

struct s_result non_block_ser_send_ack (const char *data, const int write_cr) {
	/* select-vars */
	int err;
	fd_set read_fds;
	struct timeval timeout;

	/* read-vars */
	char buf[BUFSIZ];
	int counter = 0;
	int read_count = 0;
	int la_counter = 0;
	char *tmp_buf;
	struct s_result line_array;

	line_array.result = calloc(1024, 1);

	pmesg(100, "Sending %s in nonblock... \n", data);

	if ( write_cr ) {
		if ( (write(tasks.ser_con, data, strlen(data)) != strlen(data)) || (write(tasks.ser_con, "\r", 1) != 1) )
			printf("Error sending data... (%s) \n", strerror(errno));
	} else {
		if ( (write(tasks.ser_con, data, strlen(data)) != strlen(data)) )
			printf("Error sending data... (%s) \n", strerror(errno));
	}

	FD_ZERO(&read_fds);
	FD_SET(tasks.ser_con, &read_fds);
	timeout.tv_sec=0; timeout.tv_usec=300000;
	while ((err = select(tasks.ser_con+1, &read_fds, NULL, NULL, &timeout))) {
		memset(buf, 0, sizeof(buf));
		read_count = read(tasks.ser_con, buf, BUFSIZ);

		for (counter = 0; counter < read_count; counter++) {
			if (buf[counter] == 0) buf[counter] = (unsigned char)128;
			if (buf[counter] == '\n' || buf[counter] == '\r')
				buf[counter] = 0;
		}

		if ((strcmp(buf, data) != 0) || (strcmp(buf, "") != 0)) {
			tmp_buf = calloc(strlen(buf) + 1, 1);
			strncpy(tmp_buf, buf, strlen(buf));
			tmp_buf[strlen(buf)] = 0;
			line_array.result[la_counter] = tmp_buf;
			la_counter++;
		}

		if (la_counter > 1023) {
			printf("Too many lines in result (>1024). Stopping reader.\n");
			break;
		}
	}

	for (counter = 1; counter < la_counter; counter++)
		pmesg(100, "Result%d: -%s-\n", counter, line_array.result[counter]);

	line_array.elemcount = la_counter;
	return line_array;
}


struct s_result ser_send_ack (const char *data) {
	int counter,i,la_counter = 0;
	char tmp_answer[BUFSIZ];
	struct s_result line_array;
	char *buf;

	line_array.result = calloc(1024,1);

	pmesg(100, "Sending %s...\n", data);
	if ( (write(tasks.ser_con, data, strlen(data)) != strlen(data)) || (write(tasks.ser_con, "\r", 1) != 1) )
		printf("Error sending data... (%s) \n", strerror(errno));

	do {
		memset(tmp_answer,0,sizeof(tmp_answer));
		counter = read(tasks.ser_con, tmp_answer, BUFSIZ);

		for (i = 0; i < counter; i++){
			if (tmp_answer[i] == 0) tmp_answer[i]=(unsigned char)128;
			if (tmp_answer[i] == '\n' || tmp_answer[i] == '\r')
					tmp_answer[i]=0;
		}

		/* Don't return the command itself or empty lines. */
		if ((strcmp(tmp_answer, data) == 0) || (strcmp(tmp_answer, "") == 0))
			continue;

		buf = calloc(strlen(tmp_answer) + 1,1);
		strncpy(buf, tmp_answer, strlen(tmp_answer));
		/* null terminate */
		buf[strlen(tmp_answer)]=0;
		line_array.result[la_counter] = buf;
		la_counter++;

		if (strcmp(tmp_answer, "ERROR") == 0) {
			line_array.result[0] = calloc(6,1);
			memcpy(line_array.result[0], "ERROR", 5);
			break;
		}

		if (la_counter == 1023) {
			printf("Too many lines in result! (>1024). Stopping Reader..");
			break;
		}

	} while (strcmp(tmp_answer, "OK") != 0);

	for (i = 1; i < la_counter; i++)
		pmesg(100, "Result%d: -%s-\n", i, line_array.result[i]);

	/* Need to return the line_array thingy here */
	line_array.elemcount = la_counter;
	return line_array;
}

void ser_close (void) {
	close(tasks.ser_con);
}
