#include "common.h"

/* Please see common.h for explanation of struct 'tasks' */
struct s_tasks tasks;

int main (int argc, char** argv) {
	int optresult;

	/* Please see common.h for more information about struct 'tasks' */
	tasks.ser_con = 0;
	tasks.device = strdup("/dev/ttyS0");
	tasks.speed = B19200;
	tasks.outfile = strdup("sl45c");
	tasks.infile = strdup("sl45c");
	tasks.vcard_plain = 1;
	tasks.msglevel = 0;
	tasks.rmempos = 0;
	tasks.wmempos = 0;

	printf("SL45c - Siemens SL45 Control Center & Datasuite v%s\n", VERSION);
	printf("Written by Sander Smeenk <ssmeenk@debian.org>. Released under the GPL\n");

	while (1) {
		int option_index = 0;
		static struct option long_options[] = {
			{"debug", 1, 0, 'D'},
			{"info", 0, 0, 'i'},
			{"help", 0, 0, 'h'},
			{"version", 0, 0, 'v'},
			{"device", 1, 0, 'd'},
			{"speed", 1, 0, 'e'},
			{"scan-networks", 0, 0, 'S'},
			{"last-dialed", 0, 0, 'l'},
			{"last-received", 0, 0, 'r'},
			{"missed-calls", 0, 0, 'm'},
			{"synctime", 0, 0, 't'},
			{"sim", 0, 0, 's'},
			{"read", 1, 0, 0},
			{"rmempos", 1, 0, 0},
			{"write", 1, 0, 0},
			{"wmempos", 1, 0, 0},
			{"in", 1, 0, 0},
			{"out", 1, 0, 0},
			{"dump", 0, 0, 0},
			{"csv", 0, 0, 0},
			{"smenu", 1, 0, 0},
			{0, 0, 0, 0}
		};

		optresult = getopt_long (argc, argv, "D:ihvd:e:sSlrmt", long_options, &option_index);
		if (optresult == -1)
			break;

        switch (optresult) {
			/* optresult == 0 if the long option doesn't have a short option equivalent */
	        case 0:
				if (long_options[option_index].name == "smenu") {
					tasks.runEnableSMenu = 1;
					if (optarg) {
						tasks.SMenuStyle = atoi(optarg);
					} else {
						tasks.SMenuStyle = 1;
					}
				}
				
				if (long_options[option_index].name == "sim")
					tasks.runSIMStored = 1;
				
				if (long_options[option_index].name == "csv")
					tasks.vcard_plain = 0;

				if (long_options[option_index].name == "dump") 
					tasks.runDump = 1;

				if (long_options[option_index].name == "in") {
					pmesg(100, "Using input filename %s when necessary ...\n", optarg);
					tasks.infile = optarg;
				}

				if (long_options[option_index].name == "out") {
					pmesg(100, "Using output filename %s when necessary ...\n", optarg);
					tasks.outfile = optarg;
				}
				
				if (long_options[option_index].name == "rmempos") {
					if (optarg == NULL) {
						printf("\n%s: Argument --rmempos requires numerical argument!\n", argv[0]);
						exit (0);
					} else {
						tasks.rmempos = atoi(optarg);
					}
				}

				if (long_options[option_index].name == "read")
					if ((strcmp(optarg, "bmp") != 0) && (strcmp(optarg, "mid") != 0) && (strcmp(optarg, "vcf") != 0) && (strcmp(optarg, "vcs") != 0)) {
						printf("\n%s: Argument --read requires parameter (bmp, mid, vcf or vcs)!\n", argv[0]);
						exit (0);
					} else {
						tasks.rmemtype = optarg;
						tasks.runRead = 1;
					}

				if (long_options[option_index].name == "wmempos") {
					if (optarg == NULL) {
						printf("\n%s: Argument --wmempos requires numerical argument!\n", argv[0]);
						exit(0);
					} else {
						tasks.wmempos = atoi(optarg);
					}
				}
				
				if (long_options[option_index].name == "write")
					if ((strcmp(optarg, "bmp") != 0) && (strcmp(optarg, "mid") != 0) && (strcmp(optarg, "vcf") != 0) && (strcmp(optarg, "vcs") != 0)) {
						printf("\n%s: Argument --write requires parameter (bmp, mid, vcf or vcs)!\n", argv[0]);
						exit (0);
					} else {
						tasks.wmemtype = optarg;
						tasks.runWrite = 1;
					}

				break;

	        case 'i':
				tasks.runInfo = 1;
	            break;

			case 'v':
				tasks.runVersion = 1;
				break;
				
	        case 'h':
				tasks.runHelp = 1;
	            break;

			case 'D':
				tasks.msglevel = atoi(optarg);
				pmesg(100, "Using debug level %d\n", tasks.msglevel);
				break;

			case 'd':
				if (!strstr(optarg, "dev/")) {
					printf("\n%s: Argument -d requires device parameter! (not %s)\n", argv[0], optarg);
					exit(0);
				} else {
					pmesg(100, "Set device to %s\n", optarg);
					tasks.device = optarg;
				}
				break;

			case 'e':
				switch (atoi(optarg)) {
					case 19200: tasks.speed = B19200; break;
					case 38400: tasks.speed = B38400; break;
					case 57600: tasks.speed = B57600; break;
					case 115200: tasks.speed = B115200; break;
					default:
						printf("\nOnly 19200, 38400, 57600 and 115200 are allowed speed values!\n");
						exit(0);
				}
				pmesg(100, "Serial speed set to %s\n", optarg);
				break;
				
			case 'S':
				tasks.runScan = 1;
				break;

			case 's':
				tasks.runSIMStored = 1;
				break;

			case 'l':
				tasks.runLastDialed = 1;
				break;

			case 'r':
				tasks.runLastReceived = 1;
				break;

			case 'm':
				tasks.runMissed = 1;
				break;

			case 't':
				tasks.runSync = 1;
				break;
		}
	}

	ser_open();

	if (tasks.runDump) 
		read_memory_vcf(1, 500);

	if (tasks.runSIMStored)
		show_phonebook("SM");
	
	if (tasks.runRead)
		read_memory(tasks.rmemtype, tasks.rmempos);

	if (tasks.runWrite)
		write_memory(tasks.wmemtype, tasks.wmempos);

	if (tasks.runInfo)
		information();

	if (tasks.runVersion)
		print_version();

	if (tasks.runHelp)
		print_help();

	if (tasks.runScan)
		scan_networks();

	if (tasks.runLastDialed)
		show_phonebook("LD");
	
	if (tasks.runLastReceived)
		show_phonebook("RC");

	if (tasks.runMissed)
		show_phonebook("MC");

	if (tasks.runSync)
		sync_time();

	if (tasks.runEnableSMenu)
		ena_smenu(tasks.SMenuStyle);
	
	if ( ! (tasks.runInfo || tasks.runHelp || tasks.runVersion || 
			tasks.runScan || tasks.runLastDialed || tasks.runLastReceived || 
			tasks.runMissed || tasks.runSync || tasks.runRead || 
			tasks.runWrite || tasks.runDump || tasks.runEnableSMenu || tasks.runSIMStored ) )
		print_help();

	return 0;

}
